import { ProjectModel, ProjectRequest } from 'ts-common/src/model/webSocket/project';
import { Broker } from './broker';
import { ResponseCallback, SaveState, WsFrontendCall, wsKeys, wsReturnKey, wsSubKeys } from 'ts-common';

type GetProjectResponse = (data: ProjectModel, status: string) => void;
type GetProjectsResponse = (data: ProjectModel[], status: string) => void;

function transformToProjectModel(saveStates: SaveState[]): ProjectModel[] {
  return saveStates
    .filter(ss => ss.project && ss.project.id)
    .map(
      ss =>
        ({
          id: ss.project!.id,
          name: ss.project!.name,
          parentId: ss.project!.parentId ? ss.project!.parentId : null,
          saveStates: [],
          children: [],
          sub_projects: [],
          createdAt: ss.createdAt,
          updatedAt: ss.updatedAt,
        }) as ProjectModel,
    );
}
export const wsGetProject: WsFrontendCall<{ userID: number; parentID: number }, wsReturnKey.project> = (params, callback) => {
  Broker.instance().sendMessage(
    {
      key: wsKeys.project,
      subKey: wsSubKeys.get,
      body: { userID: params.userID, parentID: params.parentID },
    },
    callback,
  );
};

export const wsGetProjects: WsFrontendCall<{ userID: number; parentID: number | null }, wsReturnKey.projects> = (params, callback) => {
  Broker.instance().sendMessage(
    {
      key: wsKeys.project,
      subKey: wsSubKeys.getAll,
      body: { parentID: params.parentID },
    },
    callback,
  );
};

export const wsCreateProject: WsFrontendCall<{ project: ProjectRequest }, wsReturnKey.project> = (params, callback) => {
  Broker.instance().sendMessage(
    {
      key: wsKeys.project,
      subKey: wsSubKeys.getAll,
      body: params.project,
    },
    callback,
  );
};

export const wsUpdateProject: WsFrontendCall<{ id: number; project: ProjectRequest }, wsReturnKey.project> = (params, callback) => {
  Broker.instance().sendMessage(
    {
      key: wsKeys.project,
      subKey: wsSubKeys.update,
      body: { queryID: params.id, ...params.project },
    },
    callback,
  );
};

export const wsDeleteProject: WsFrontendCall<{ id: number }, wsReturnKey.project> = (params, callback) => {
  Broker.instance().sendMessage(
    {
      key: wsKeys.project,
      subKey: wsSubKeys.delete,
      body: { id: params.id },
    },
    callback,
  );
};

export const wsProjectSubscription = (callback: ResponseCallback<wsReturnKey.project>) => {
  const id = Broker.instance().subscribe(callback, wsReturnKey.project);
  return () => {
    Broker.instance().unSubscribe(wsReturnKey.project, id);
  };
};
